==> virtualfs-1.13.src.tar.gz: virtualfs-1.13/litefsutil.cc <==
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include "litefs.h"

int litefsd_server(const char *sockn)
{
	int ret = -1;
	unlink (sockn);
	int fd =  socket (AF_UNIX,SOCK_STREAM,0);
	if (fd == -1){
		perror("socket server");
	}else{
		struct sockaddr_un un;
		un.sun_family = AF_UNIX;
		strcpy (un.sun_path,sockn);
		if (bind(fd,(struct sockaddr*)&un,sizeof(un))==-1){
			perror("bind");
		}else{
			chmod (sockn,0600);
			int code = listen (fd,10);
			if (code == -1){
				perror ("listen");
			}else{
				ret = fd;
			}	
		}
	}
	return ret;
}

int litefsd_tcpserver(const char *portstr)
{
	int ret = -1;
	int port = -1;
	struct servent *serv = getservbyname (portstr,"tcp");
	if (serv != NULL){
		port = serv->s_port;
	}else if (isdigit(portstr[0])){
		port = atoi(portstr);
	}
	if (port == -1){
		fprintf (stderr,"tcp port unknown %s\n",portstr);
	}else{
		char myhost[PATH_MAX];
		gethostname (myhost, sizeof(myhost));
		struct hostent *h = (struct hostent *) gethostbyname(myhost);
		if (h == NULL){
			fprintf (stderr,"gethostbyname(%s) failed",myhost);
		}else{
			struct sockaddr_in sin;
			sin.sin_family = h->h_addrtype;
			memcpy  (&sin.sin_addr,h->h_addr, h->h_length);
			sin.sin_port = htons(port);

			int i;
			for (i=0; i<5; i++){
				int listen_handle = socket (AF_INET, SOCK_STREAM, 0);
				sin.sin_addr.s_addr = INADDR_ANY;
				int opt = 1;
				if (listen_handle == -1){
					fprintf (stderr,"listen_handle %d(%s)\n"
						,errno
						,strerror(errno));
				}else if (setsockopt(listen_handle,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt))==-1){
					fprintf (stderr,"Can't set socket option SO_REUSEADDR (%s)\n"
						,strerror (errno));
				}else if (bind (listen_handle,(struct sockaddr *) &sin, sizeof (sin)) == -1){
					fprintf (stderr,"bind %d(%s)\n",errno
						,strerror(errno));
				}else if (listen (listen_handle,5) == -1){
					fprintf (stderr,"listen %d(%s)\n",errno
						,strerror(errno));
					break;
				}else{
					ret = listen_handle;
					break;
				}
				close (listen_handle);
				listen_handle = -1;
				if (i < 5) sleep (i*5);
			}
		}
	}
	return ret;
}

int litefsd_sendfd(int sock, int fd)
{
	LITE_REPLY rep;
	rep.data = fd;
	rep.status = LITE_STATUS_HANDLE;
	rep.len = 0;
	return write (sock,&rep,sizeof(rep));
}

static void litefsd_sendfail (int fd, int errcode)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_FAIL;
	rep.errcode = errcode;
	rep.len = 0;
	write (fd,&rep,sizeof(rep));
}
void litefsd_sendfail (int fd)
{
	litefsd_sendfail (fd,EINVAL);
}
void litefsd_senderrno(int fd)
{
	litefsd_sendfail (fd,errno);
}

void litefsd_sendok (int fd, int len)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_OK;
	rep.errcode = 0;
	rep.data = len;
	rep.len = 0;
	write (fd,&rep,sizeof(rep));
}
void litefsd_sendok (int fd)
{
	litefsd_sendok (fd,0);
}
void litefsd_sendseek(int fd, off_t off)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_SEEK;
	rep.data = off;
	rep.errcode = 0;
	rep.len = 0;
	write (fd,&rep,sizeof(rep));
}
void litefsd_sendhandle (int fd, int handle)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_HANDLE;
	rep.data = handle;
	rep.errcode = 0;
	rep.len = 0;
	write (fd,&rep,sizeof(rep));
}

void litefsd_sendstat (int fd, struct stat &st)
{
	LITE_REPLY_DATA rep;
	rep.status = LITE_STATUS_STAT;
	memcpy (rep.buf,&st,sizeof(struct stat));
	rep.len = sizeof(struct stat);
	write (fd,&rep,sizeof(LITE_REPLY)+rep.len);
}

==> virtualfs-1.21.src.tar.gz: virtualfs-1.21/litefsutil.cc <==
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include "litefs.h"

int litefsd_server(const char *sockn)
{
	int ret = -1;
	unlink (sockn);
	int fd =  socket (AF_UNIX,SOCK_STREAM,0);
	if (fd == -1){
		perror("socket server");
	}else{
		struct sockaddr_un un;
		un.sun_family = AF_UNIX;
		strcpy (un.sun_path,sockn);
		if (bind(fd,(struct sockaddr*)&un,sizeof(un))==-1){
			perror("bind");
		}else{
			chmod (sockn,0600);
			int code = listen (fd,10);
			if (code == -1){
				perror ("listen");
			}else{
				ret = fd;
			}	
		}
	}
	return ret;
}

int litefsd_tcpserver(const char *portstr)
{
	int ret = -1;
	int port = -1;
	struct servent *serv = getservbyname (portstr,"tcp");
	if (serv != NULL){
		port = serv->s_port;
	}else if (isdigit(portstr[0])){
		port = atoi(portstr);
	}
	if (port == -1){
		fprintf (stderr,"tcp port unknown %s\n",portstr);
	}else{
		char myhost[PATH_MAX];
		gethostname (myhost, sizeof(myhost));
		struct hostent *h = (struct hostent *) gethostbyname(myhost);
		if (h == NULL){
			fprintf (stderr,"gethostbyname(%s) failed",myhost);
		}else{
			struct sockaddr_in sin;
			sin.sin_family = h->h_addrtype;
			memcpy  (&sin.sin_addr,h->h_addr, h->h_length);
			sin.sin_port = htons(port);

			int i;
			for (i=0; i<5; i++){
				int listen_handle = socket (AF_INET, SOCK_STREAM, 0);
				sin.sin_addr.s_addr = INADDR_ANY;
				int opt = 1;
				if (listen_handle == -1){
					fprintf (stderr,"listen_handle %d(%s)\n"
						,errno
						,strerror(errno));
				}else if (setsockopt(listen_handle,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt))==-1){
					fprintf (stderr,"Can't set socket option SO_REUSEADDR (%s)\n"
						,strerror (errno));
				}else if (bind (listen_handle,(struct sockaddr *) &sin, sizeof (sin)) == -1){
					fprintf (stderr,"bind %d(%s)\n",errno
						,strerror(errno));
				}else if (listen (listen_handle,5) == -1){
					fprintf (stderr,"listen %d(%s)\n",errno
						,strerror(errno));
					break;
				}else{
					ret = listen_handle;
					break;
				}
				close (listen_handle);
				listen_handle = -1;
				if (i < 5) sleep (i*5);
			}
		}
	}
	return ret;
}

int litefsd_sendfd(LITEFSD_CTX &ctx, int fd)
{
	LITE_REPLY rep;
	rep.data = fd;
	rep.status = LITE_STATUS_HANDLE;
	rep.len = 0;
	rep.client_handle = ctx.client_handle;
	return write (ctx.fd,&rep,sizeof(rep));
}

static void litefsd_sendfail (LITEFSD_CTX &ctx, int errcode)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_FAIL;
	rep.errcode = errcode;
	rep.len = 0;
	rep.client_handle = ctx.client_handle;
	write (ctx.fd,&rep,sizeof(rep));
}
void litefsd_sendfail (LITEFSD_CTX &ctx)
{
	litefsd_sendfail (ctx,EINVAL);
}
void litefsd_senderrno(LITEFSD_CTX &ctx)
{
	litefsd_sendfail (ctx,errno);
}

void litefsd_sendok (LITEFSD_CTX &ctx, int len)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_OK;
	rep.errcode = 0;
	rep.data = len;
	rep.len = 0;
	rep.client_handle = ctx.client_handle;
	write (ctx.fd,&rep,sizeof(rep));
}
void litefsd_sendok (LITEFSD_CTX &ctx)
{
	litefsd_sendok (ctx,0);
}
void litefsd_sendseek(LITEFSD_CTX &ctx, off_t off)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_SEEK;
	rep.data = off;
	rep.errcode = 0;
	rep.len = 0;
	rep.client_handle = ctx.client_handle;
	write (ctx.fd,&rep,sizeof(rep));
}
void litefsd_sendhandle (LITEFSD_CTX &ctx, int handle)
{
	LITE_REPLY rep;
	rep.status = LITE_STATUS_HANDLE;
	rep.data = handle;
	rep.errcode = 0;
	rep.len = 0;
	rep.client_handle = ctx.client_handle;
	write (ctx.fd,&rep,sizeof(rep));
}

void litefsd_sendstat (LITEFSD_CTX &ctx, struct stat &st)
{
	LITE_REPLY_DATA rep;
	rep.status = LITE_STATUS_STAT;
	memcpy (rep.buf,&st,sizeof(struct stat));
	rep.len = sizeof(struct stat);
	rep.client_handle = ctx.client_handle;
	write (ctx.fd,&rep,sizeof(LITE_REPLY)+rep.len);
}
